from Plugins.Plugin import PluginDescriptor
from GraphMultiEpg import GraphMultiEPG
from Screens.ChannelSelection import BouquetSelector
from enigma import eServiceCenter, eServiceReference
from ServiceReference import ServiceReference

Session = None
Servicelist = None

bouquetSel = None
epg_bouquet = None
all_bouquets = [ ]
dlg_stack = [ ]
current_bouquet = None

def zapToService(service):
	if not service is None:
		if Servicelist.getRoot() != epg_bouquet: #already in correct bouquet?
			Servicelist.clearPath()
			if Servicelist.bouquet_root != epg_bouquet:
				Servicelist.enterPath(Servicelist.bouquet_root)
			Servicelist.enterPath(epg_bouquet)
		Servicelist.setCurrentSelection(service) #select the service in Servicelist
		Servicelist.zap()

def getBouquetServices(bouquet):
	services = [ ]
	Servicelist = eServiceCenter.getInstance().list(bouquet)
	if not Servicelist is None:
		while True:
			service = Servicelist.getNext()
			if not service.valid(): #check if end of list
				break
			if service.flags & (eServiceReference.isDirectory | eServiceReference.isMarker): #ignore non playable services
				continue
			services.append(ServiceReference(service))
	return services

def cleanup():
	global Session
	Session = None
	global Servicelist
	Servicelist = None

def closed(ret=False):
	closedScreen = dlg_stack.pop()
	global bouquetSel
	if bouquetSel and closedScreen == bouquetSel:
		bouquetSel = None
	dlgs=len(dlg_stack)
	if ret and dlgs > 0: # recursive close wished
		dlg_stack[dlgs-1].close(dlgs > 1)
	if dlgs <= 0:
		cleanup()

def openBouquetEPG(bouquet):
	services = getBouquetServices(bouquet)
	if len(services):
		global epg_bouquet
		epg_bouquet = bouquet
		dlg_stack.append(Session.openWithCallback(closed, GraphMultiEPG, services, zapToService, changeBouquetCB))
		return True
	return False

def changeBouquetCB(direction, epg):
	if bouquetSel:
		if direction > 0:
			bouquetSel.down()
		else:
			bouquetSel.up()
		bouquet = bouquetSel.getCurrent()
		services = getBouquetServices(bouquet)
		if len(services):
			global epg_bouquet
			epg_bouquet = bouquet
			epg.setServices(services)
	else:
		for i in range(len(all_bouquets)):
			if all_bouquets[i][1] == current_bouquet:
				break
		i = i + 1
 		if direction > 0:
			if i == len(all_bouquets):
				j = 0
			else:
				j = i
		else:
			if i == 1:
				j = len(all_bouquets)
				j = j - 1
			else:
				j = i - 2
		services = getBouquetServices(all_bouquets[j][1])                                
		if len(services):
			global epg_bouquet
			global current_bouquet
			current_bouquet = epg_bouquet = all_bouquets[j][1]
			epg.setServices(services)

def main(session, servicelist, **kwargs):
	global Session
	global all_bouquets
	global current_bouquet
	Session = session
	global Servicelist
	Servicelist = servicelist
        current_bouquet = Servicelist.getRoot()
	bouquets = Servicelist.getBouquetList()
	del all_bouquets[:] 
	#f = open('/tmp/one.dat', 'a')
	#f.write("Len:")
	for i in range(len(bouquets)):
		services = getBouquetServices(bouquets[i][1])
		if len(services):
			all_bouquets.append(bouquets[i])
	services = getBouquetServices(current_bouquet)
	#f.write(str(len(services)))
	#f.close()
	if bouquets is None:
		cnt = 0
	else:
		cnt = len(bouquets)
	if len(services):
		cnt = 0
	if cnt > 1: # show bouquet list
		global bouquetSel
		bouquetSel = Session.openWithCallback(closed, BouquetSelector, bouquets, openBouquetEPG, enableWrapAround=True)
		dlg_stack.append(bouquetSel)
	elif cnt == 1:
		if not openBouquetEPG(bouquets[0][1]):
			cleanup()
	elif cnt == 0:
		if not openBouquetEPG(current_bouquet):
			cleanup()

def Plugins(**kwargs):
	name = _("Graphical Multi EPG")
	descr = _("A graphical EPG for all services of an specific bouquet")
 	return [ PluginDescriptor(name=name, description=descr, where = PluginDescriptor.WHERE_EVENTINFO, fnc=main),
	  PluginDescriptor(name=name, description=descr, where = PluginDescriptor.WHERE_EXTENSIONSMENU, fnc=main) ]
